@@ -13,6 +13,7 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
+from calculate.core.server.loaded_methods import LoadedMethods
 
 from soaplib.serializers.primitive import String, Integer, Boolean, string_encoding
 from soaplib.serializers.clazz import Array, ClassSerializer
@@ -127,7 +128,7 @@ class ChoiceValue(DataVarsSerializer):
                 if items:
                     self.values, self.comments = zip(*items)
                 else:
-                    self.values, self.comments = [],[]
+                    self.values, self.comments = [], []
                 self.values = self.arrayString(self.values)
                 self.comments = self.arrayString(self.comments)
             if varObj.mode == 'w':
@@ -158,10 +159,15 @@ class Table(DataVarsSerializer):
     values = Array(ChoiceValue)
     onClick = String
     addAction = String
+    # шаг на который нужно прейти при выполнении onClick
+    step = String
+    # количество записей в таблице
+    records = String
 
     def __init__(self, dv=None, briefmode=None,
                  varObj=None, head=None, body=None, values=None,
-                 fields=None, onClick=None, addAction=None):
+                 fields=None, onClick=None, addAction=None, step=None,
+                 records=None):
         super(Table, self).__init__()
         if dv:
             self.head = []
@@ -196,6 +202,8 @@ class Table(DataVarsSerializer):
             self.values = values
             self.onClick = onClick
             self.addAction = addAction
+            self.step = step or "0"
+            self.records = records or "0"
 
 
 class Option(DataVarsSerializer):
@@ -224,6 +232,7 @@ class Field(DataVarsSerializer):
     opt = Option  # opt for cmdline
     help = String  # help for cmdline (GUI?)
     element = String  # type element
+    guitype = String # addon guitype
     choice = Array(String)  # value (combobox,comboedit)
     listvalue = Array(String)  # current listvalue
     default = Boolean  # default value or False
@@ -247,7 +256,8 @@ class Field(DataVarsSerializer):
                                   varObj.syntax)
             self.help = varObj.help
             self.element = varObj.element or self.elementByType(self.type)
-            self.label = varObj.label or varObj.name
+            self.label = str(varObj.label or varObj.name)
+            self.guitype = varObj.guitype
             if (not onlyhelp or varObj.syntax or self.type == "table" or
                         self.type == "bool"):
                 if inbrief:
@@ -352,7 +362,7 @@ class GroupField(DataVarsSerializer):
                             varname).wasSet:
                         self.fields.append(Field(name="expert",
                                                  element="expert",
-                                                 label=info['expert_label'],
+                                                 label=str(info['expert_label']),
                                                  value="open"))
                         for varname_ in info['expert']:
                             inbrief = brief and varname_ in info['brief_force']
@@ -375,7 +385,7 @@ class GroupField(DataVarsSerializer):
                     if not onlyhelp:
                         self.fields.append(Field(name="expert",
                                                  element="expert",
-                                                 label=info['expert_label'],
+                                                 label=str(info['expert_label']),
                                                  value="close",
                                                  onlyhelp=onlyhelp))
         else:
@@ -400,6 +410,7 @@ class ViewInfo(DataVarsSerializer):
             brief = viewparams.brief if brief is None else brief
             onlyhelp = viewparams.onlyhelp
         self.has_brief = has_brief
+
         if datavars:
             self.groups = []
             varGroups = datavars.getGroups()
@@ -423,9 +434,22 @@ class ViewInfo(DataVarsSerializer):
                                                   onlyhelp=onlyhelp))
                     if groupInfo['custom_buttons']:
                         for but in groupInfo['custom_buttons']:
+                            if len(but) > 4:
+                                listval = but[4]
+                            else:
+                                listval = None
+                            if len(but) > 5:
+                                f = but[5]
+                                if f is None or callable(f) and f(datavars.Get):
+                                    enable = True
+                                else:
+                                    enable = False
+                            else:
+                                enable = True
                             self.groups[-1].fields.append(Field(
                                 name=but[0], label=but[1], value=but[2],
-                                element=but[3]))
+                                element=but[3], listvalue=listval,
+                                guitype=None if enable else "readonly"))
 
         else:
             self.groups = groups
@@ -460,6 +484,8 @@ class ViewParams(ClassSerializer):
     expert = Boolean  # request expert variables
     brief = Boolean  # request brief variables
     onlyhelp = Boolean  # request params for only help
+    help_set = Boolean  # set cl_help_set to on
+    dispatch_usenew = Boolean  # set cl_dispatch_conf to usenew
     clienttype = String  # type of client "gui","console"
 
 
@@ -530,7 +556,8 @@ class CoreWsdl(CoreServiceInterface):
 
     def callAction(self, sid, info, logicClass=None,
                    method_name=None, actionClass=None,
-                   callbackRefresh=lambda sid, dv: True):
+                   callbackRefresh=lambda sid, dv: True,
+                   invalidators=None, depend_methods=()):
         """
         Общий алгоритм вызова действия
         """
@@ -553,7 +580,8 @@ class CoreWsdl(CoreServiceInterface):
                 checkall = info.CheckAll
             errors = map(lambda x: ReturnedMessage(**x),
                          dv.checkGroups(info,
-                                        allvars=checkall or not checkonly))
+                                        allvars=checkall or not checkonly,
+                                        invalidators=invalidators))
             # if dv.Get('cl_env_debug_set') == 'on':
             #    dv.printGroup(info)
             if errors:
@@ -579,10 +607,20 @@ class CoreWsdl(CoreServiceInterface):
             returnmess.type = "pid"
             returnmess.message = pid
             self.clear_cache(sid, method_name)
+            cache_cleared = [method_name]
+            clear_list = list(LoadedMethods.methodDepends[method_name])
+            while clear_list:
+                method_clear_name = clear_list.pop()
+                if method_clear_name not in cache_cleared:
+                    clear_list.extend(
+                        LoadedMethods.methodDepends[method_clear_name])
+                    self.clear_cache_method(method_clear_name)
+                    cache_cleared.append(method_clear_name)
             dv = None
             return [returnmess]
         finally:
             if dv:
+                # print "Set cache", dv.Get('ur_unix_group_name_exists')
                 self.set_cache(sid, method_name, "vars", dv, smart=False)
 
     def fixInstallLocalization(self, sid, dv):
