@@ -14,11 +14,14 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+from collections import defaultdict
+
 
 class LoadedMethods(object):
     conMethods = {}
     guiMethods = {}
     rightsMethods = {}
+    methodDepends = defaultdict(list)
 
     # add needed rights for call method
     @classmethod
@@ -32,7 +35,7 @@ class LoadedMethods(object):
     # add needed rights for call method
     @classmethod
     def core_method(cls, gui=False, category=None, title=None, image=None,
-                    command=None, rights=(), user=False):
+                    command=None, rights=(), user=False, depends=()):
         def wrapper(function):
             if gui:
                 if category not in cls.guiMethods.keys():
@@ -40,6 +43,10 @@ class LoadedMethods(object):
                 cls.guiMethods[category].append(function.__name__)
                 cls.guiMethods[category].append(title)
                 cls.guiMethods[category].append(image)
+                if callable(gui):
+                    cls.guiMethods[category].append(gui)
+                else:
+                    cls.guiMethods[category].append(None)
 
             if command:
                 if command not in cls.conMethods.keys():
@@ -48,6 +55,8 @@ class LoadedMethods(object):
                 cls.conMethods[command].append(user)
                 if title:
                     cls.conMethods[command].append(title)
+            for depend in depends:
+                cls.methodDepends[depend].append(function.__name__)
 
             if rights:
                 cls.rightsMethods[function.__name__] = rights
