@@ -22,6 +22,8 @@ from os import path
 from fcntl import ioctl
 from array import array
 import threading
+import argparse
+import re
 
 from calculate.lib.utils.colortext import get_terminal_print
 from calculate.lib.utils.colortext.palette import TextState
@@ -40,7 +42,7 @@ from gen_pid import ProcessStatus
 from methods_func import (get_method_argparser, collect_object,
                           check_result_msg, get_param_pwd, _print,
                           display_error)
-from api_types import ViewInfo
+from api_types import ViewInfo, ViewParams
 from cert_cmd import parse
 from methods_func import GotErrorField
 from func import shortTraceback, CommonMethods, CommonLink
@@ -198,26 +200,39 @@ def local_method(metaclass, args, unknown_args):
     method_name = args.method
     method_view_name = method_name + '_view'
     if args.method and args.help:
-        view_obj = ViewInfo()
-        view_obj.step = None
-        view_obj.expert = True
-        view_obj.brief = None
-        view_obj.onlyhelp = True
-        try:
-            view = getattr(metaObject, method_view_name)(0, view_obj)
-        except AttributeError:
-            colorPrint.printERROR(_('Method not found: ') + method_view_name)
-            return 1
-        try:
-            method_parser = get_method_argparser(view, args, cl_core=True)
-        except Exception:
-            # import traceback
-            # for i in apply(traceback.format_exception, sys.exc_info()):
-            #    sys.stderr.write(i)
-            # sys.stderr.flush()
-            metaObject.clear_cache(0, method_name)
-            return 1
-        method_parser.print_help()
+        force_param = args.no_questions or has_force_arg(unknown_args)
+        while True:
+            view_obj = ViewParams()
+            view_obj.step = None
+            view_obj.expert = True
+            view_obj.brief = None
+            view_obj.onlyhelp = True
+            view_obj.help_set = True
+            view_obj.dispatch_usenew = \
+                force_param
+            try:
+                view = getattr(metaObject, method_view_name)(0, view_obj)
+            except AttributeError:
+                colorPrint.printERROR(_('Method not found: ') + method_view_name)
+                return 1
+            try:
+                method_parser = get_method_argparser(view, args, cl_core=True)
+            except Exception:
+                # import traceback
+                # for i in apply(traceback.format_exception, sys.exc_info()):
+                #    sys.stderr.write(i)
+                # sys.stderr.flush()
+                metaObject.clear_cache(0, method_name)
+                return 1
+            _unknown_args = method_parser.fixBoolVariables(unknown_args)
+            _args, _unknown_args = method_parser.parse_known_args(_unknown_args)
+            if (view_obj.dispatch_usenew == _args.no_questions
+                or args.no_questions):
+                method_parser.print_help()
+                break
+            else:
+                force_param = _args.no_questions
+                metaObject.clear_cache(0, method_name)
     else:
         try:
             call_method(metaObject, args, unknown_args, colorPrint)
@@ -248,23 +263,41 @@ def call_method(metaObject, args, unknown_args, colorPrint):
     metaObject.gui_progress = args.gui_progress
     metaObject.gui_warning = args.gui_warning
     metaObject.no_questions = False
-    view_obj = ViewInfo()
-    view_obj.step = None
-    view_obj.expert = True
-    view_obj.brief = None
-    view_obj.onlyhelp = True
+    view = None
+    method_parser = None
+
+    dispatch_usenew = args.no_questions or has_force_arg(unknown_args)
+    while True:
+        view_obj = ViewInfo()
+        view_obj.step = None
+        view_obj.expert = True
+        view_obj.brief = None
+        view_obj.onlyhelp = True
+        view_obj.help_set = False
+        view_obj.dispatch_usenew = dispatch_usenew
+
+        try:
+            view = getattr(metaObject, method_view_name)(0, view_obj)
+        except AttributeError:
+            colorPrint.printERROR(_('Method not found: ') + method_name)
+            return None
+        method_parser = get_method_argparser(view, args, cl_core=True)
+
+        _unknown_args = method_parser.fixBoolVariables(unknown_args)
+        _args, _unknown_args = method_parser.parse_known_args(_unknown_args)
+        if (view_obj.dispatch_usenew == _args.no_questions or
+                args.no_questions):
+            break
+        else:
+            dispatch_usenew = _args.no_questions
+            metaObject.clear_cache(0, method_name)
+
+    no_questions = dispatch_usenew
 
-    try:
-        view = getattr(metaObject, method_view_name)(0, view_obj)
-    except AttributeError:
-        colorPrint.printERROR(_('Method not found: ') + method_name)
-        return None
-    method_parser = get_method_argparser(view, args, cl_core=True)
     param_object = create_param_object(view)
     try:
         unknown_args = method_parser.fixBoolVariables(unknown_args)
         args, unknown_args = method_parser.parse_known_args(unknown_args)
-        no_questions = args.no_questions
         metaObject.no_questions = no_questions
     except SystemExit:
         return 1
@@ -312,6 +345,8 @@ def call_method(metaObject, args, unknown_args, colorPrint):
         view_obj.expert = True
         view_obj.brief = True
         view_obj.onlyhelp = False
+        view_obj.help_set = False
+        view_obj.dispatch_usenew = dispatch_usenew
         try:
             view = getattr(metaObject, method_view_name)(0, view_obj)
         except AttributeError:
@@ -336,25 +371,40 @@ def call_method(metaObject, args, unknown_args, colorPrint):
                 return None
 
     setattr(param_object, 'CheckOnly', False)
+    method_result = []
     try:
-        method_result = getattr(metaObject, method_name)(0, param_object)
+        check_res = {}
+        while True:
+            method_result = getattr(metaObject, method_name)(0, param_object)
+            if not method_result:
+                colorPrint.printERROR(_('method unavailable'))
+                return None
+
+            if method_result[0].type and method_result[0].type != "pid":
+                check_res = check_result_msg(method_result, view, check_res,
+                                             args)
+                if not check_res:
+                    return None
+                else:
+                    param_object = get_param_pwd(check_res, view,
+                                                 param_object,
+                                                 stdin_passwd=stdin_passwd)
+            else:
+                break
     except VariableError, e:
         _print(e)
         return None
-    if not method_result:
-        colorPrint.printERROR(_('method unavailable'))
-        return None
-    for ReturnedMessage in method_result:
-        if ReturnedMessage.type and ReturnedMessage.type != "pid":
-            display_error(ReturnedMessage, args, view.groups)
-            # params_text = ''
-            # for Group in view.groups:
-            #    for field in Group.fields:
-            #        if field.name == ReturnedMessage.field:
-            #            params_text += getErrorOnParam(args, field)
-            # colorPrint.printERROR('\r' + params_text % \
-            #                      str(ReturnedMessage.message))
-            return None
+    #for ReturnedMessage in method_result:
+    #    if ReturnedMessage.type and ReturnedMessage.type != "pid":
+    #        display_error(ReturnedMessage, args, view.groups)
+    #        # params_text = ''
+    #        # for Group in view.groups:
+    #        #    for field in Group.fields:
+    #        #        if field.name == ReturnedMessage.field:
+    #        #            params_text += getErrorOnParam(args, field)
+    #        # colorPrint.printERROR('\r' + params_text % \
+    #        #                      str(ReturnedMessage.message))
+    #        return None
     return method_result
 
 
@@ -590,3 +640,27 @@ class Methods(LocalCall.Common, object):
 
     def __init__(self):
         LocalCall.Common.__init__(self, False, False, False, False)
+
+
+def has_force_arg(args):
+    """
+    Содержат ли аргумент force. Предварительное определение, так как на 100%
+    невозможно определить является ли -sf двумя опциями -s,-f или это одна
+    опция -s со значением "f"
+    :param args:
+    :return:
+    """
+    force_parser = argparse.ArgumentParser(add_help=False)
+    force_parser.add_argument(
+        '-f', '--force', default=False, dest='force', action="store_true")
+    _args, _drop = force_parser.parse_known_args(args)
+
+    if _args.force:
+        return True
+
+    re_force = re.compile("^--force|-[a-zA-Z0-9]*f[a-zA-Z0-9]*$")
+    for arg in args:
+        if re_force.search(arg):
+            return True
+    else:
+        return False
